"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc2) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc2 = __getOwnPropDesc(from, key)) || desc2.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  buildSchema: () => buildSchema
});
module.exports = __toCommonJS(src_exports);
var import_drizzle_orm7 = require("drizzle-orm");
var import_mysql_core3 = require("drizzle-orm/mysql-core");
var import_pg_core3 = require("drizzle-orm/pg-core");
var import_sqlite_core3 = require("drizzle-orm/sqlite-core");
var import_graphql7 = require("graphql");

// src/util/builders/mysql.ts
var import_drizzle_orm4 = require("drizzle-orm");
var import_mysql_core2 = require("drizzle-orm/mysql-core");
var import_graphql4 = require("graphql");

// src/util/builders/common.ts
var import_drizzle_orm3 = require("drizzle-orm");
var import_graphql3 = require("graphql");
var import_graphql_parse_resolve_info = require("graphql-parse-resolve-info");

// src/util/case-ops/index.ts
var camelize = (input) => input.toLocaleLowerCase();
var pascalize = (input) => input.length ? `${input[0].toLocaleUpperCase()}${input.length > 1 ? input.slice(1, input.length) : ""}` : input;

// src/util/data-mappers/index.ts
var import_drizzle_orm = require("drizzle-orm");
var import_graphql = require("graphql");
var remapToGraphQLCore = (key, value, relations) => {
  if (value instanceof Date)
    return value.toISOString();
  if (value instanceof Buffer)
    return Array.from(value);
  if (typeof value === "bigint")
    return value.toString();
  if (Array.isArray(value)) {
    if (relations?.[key])
      return remapToGraphQLArrayOutput(value);
    return value.map((arrVal) => remapToGraphQLCore(key, arrVal));
  }
  if (typeof value === "object") {
    if (relations?.[key])
      return remapToGraphQLSingleOutput(value);
    return JSON.stringify(value);
  }
  return value;
};
var remapToGraphQLSingleOutput = (queryOutput, relations) => {
  for (const [key, value] of Object.entries(queryOutput)) {
    if (value === void 0 || value === null) {
      delete queryOutput[key];
    } else {
      queryOutput[key] = remapToGraphQLCore(key, value, relations);
    }
  }
  return queryOutput;
};
var remapToGraphQLArrayOutput = (queryOutput, relations) => {
  for (const entry of queryOutput)
    remapToGraphQLSingleOutput(entry, relations);
  return queryOutput;
};
var remapFromGraphQLCore = (value, column, columnName) => {
  switch (column.dataType) {
    case "date": {
      const formatted = new Date(value);
      if (Number.isNaN(formatted.getTime()))
        throw new import_graphql.GraphQLError(`Field '${columnName}' is not a valid date!`);
      return formatted;
    }
    case "buffer": {
      if (!Array.isArray(value)) {
        throw new import_graphql.GraphQLError(`Field '${columnName}' is not an array!`);
      }
      return Buffer.from(value);
    }
    case "json": {
      try {
        return JSON.parse(value);
      } catch (e) {
        throw new import_graphql.GraphQLError(
          `Invalid JSON in field '${columnName}':
${e instanceof Error ? e.message : "Unknown error"}`
        );
      }
    }
    case "array": {
      if (!Array.isArray(value)) {
        throw new import_graphql.GraphQLError(`Field '${columnName}' is not an array!`);
      }
      return value;
    }
    case "bigint": {
      try {
        return BigInt(value);
      } catch (error) {
        throw new import_graphql.GraphQLError(`Field '${columnName}' is not a BigInt!`);
      }
    }
    default: {
      return value;
    }
  }
};
var remapFromGraphQLSingleInput = (queryInput, table) => {
  for (const [key, value] of Object.entries(queryInput)) {
    if (value === void 0) {
      delete queryInput[key];
    } else {
      const column = (0, import_drizzle_orm.getTableColumns)(table)[key];
      if (!column)
        throw new import_graphql.GraphQLError(`Unknown column: ${key}`);
      if (value === null && column.notNull) {
        delete queryInput[key];
        continue;
      }
      queryInput[key] = remapFromGraphQLCore(value, column, key);
    }
  }
  return queryInput;
};
var remapFromGraphQLArrayInput = (queryInput, table) => {
  for (const entry of queryInput)
    remapFromGraphQLSingleInput(entry, table);
  return queryInput;
};

// src/util/type-converter/index.ts
var import_drizzle_orm2 = require("drizzle-orm");
var import_mysql_core = require("drizzle-orm/mysql-core");
var import_pg_core = require("drizzle-orm/pg-core");
var import_sqlite_core = require("drizzle-orm/sqlite-core");
var import_graphql2 = require("graphql");
var columnToGraphQLCore = (column) => {
  switch (column.dataType) {
    case "boolean":
      return { type: import_graphql2.GraphQLBoolean, description: "Boolean" };
    case "json":
      return { type: import_graphql2.GraphQLString, description: "JSON" };
    case "date":
      return { type: import_graphql2.GraphQLString, description: "Date" };
    case "string":
      return { type: import_graphql2.GraphQLString, description: "String" };
    case "bigint":
      return { type: import_graphql2.GraphQLString, description: "BigInt" };
    case "number":
      return (0, import_drizzle_orm2.is)(column, import_pg_core.PgInteger) || (0, import_drizzle_orm2.is)(column, import_pg_core.PgSerial) || (0, import_drizzle_orm2.is)(column, import_mysql_core.MySqlInt) || (0, import_drizzle_orm2.is)(column, import_mysql_core.MySqlSerial) || (0, import_drizzle_orm2.is)(column, import_sqlite_core.SQLiteInteger) ? { type: import_graphql2.GraphQLInt, description: "Integer" } : { type: import_graphql2.GraphQLFloat, description: "Float" };
    case "buffer":
      return { type: new import_graphql2.GraphQLList(new import_graphql2.GraphQLNonNull(import_graphql2.GraphQLInt)), description: "Buffer" };
    case "array": {
      const innerType = columnToGraphQLCore(column.baseColumn);
      return {
        type: new import_graphql2.GraphQLList(new import_graphql2.GraphQLNonNull(innerType.type)),
        description: `Array<${innerType.description}>`
      };
    }
    case "custom":
    default:
      throw new Error(`Type ${column.dataType} is not implemented!`);
  }
};
var drizzleColumnToGraphQLType = (column, forceNullable = false, defaultIsNullable = false) => {
  const typeDesc = columnToGraphQLCore(column);
  const noDesc = ["string", "boolean", "number"];
  if (noDesc.find((e) => e === column.dataType))
    delete typeDesc.description;
  if (forceNullable)
    return typeDesc;
  if (column.notNull && !(defaultIsNullable && (column.hasDefault || column.defaultFn))) {
    return {
      type: new import_graphql2.GraphQLNonNull(typeDesc.type),
      description: typeDesc.description
    };
  }
  return typeDesc;
};

// src/util/builders/common.ts
var extractSelectedColumnsFromNode = (info, table) => {
  if (!info.selectionSet)
    return {};
  const tableColumns = (0, import_drizzle_orm3.getTableColumns)(table);
  const selectedColumns = [];
  for (const columnSelection of info.selectionSet.selections) {
    if (columnSelection.kind !== import_graphql3.Kind.FIELD || !tableColumns[columnSelection.name.value])
      continue;
    selectedColumns.push([columnSelection.name.value, true]);
  }
  return Object.fromEntries(selectedColumns);
};
var extractSelectedColumnsSQLFormat = (info, queryName, table) => {
  const tableSelection = info.operation.selectionSet.selections.find(
    (e) => e.kind === import_graphql3.Kind.FIELD && e.name.value === queryName
  );
  const selectedColumns = [];
  if (!tableSelection || !tableSelection.selectionSet)
    throw new import_graphql3.GraphQLError("Received empty column selection!");
  for (const columnSelection of tableSelection.selectionSet.selections) {
    if (columnSelection.kind !== import_graphql3.Kind.FIELD)
      continue;
    selectedColumns.push([columnSelection.name.value, table[columnSelection.name.value]]);
  }
  return Object.fromEntries(selectedColumns);
};
var innerOrder = new import_graphql3.GraphQLInputObjectType({
  name: "InnerOrder",
  fields: {
    direction: {
      type: new import_graphql3.GraphQLNonNull(
        new import_graphql3.GraphQLEnumType({
          name: "OrderDirection",
          description: "Order by direction",
          values: {
            asc: {
              value: "asc",
              description: "Ascending order"
            },
            desc: {
              value: "desc",
              description: "Descending order"
            }
          }
        })
      )
    },
    priority: { type: new import_graphql3.GraphQLNonNull(import_graphql3.GraphQLInt), description: "Priority of current field" }
  }
});
var generateColumnFilterValues = (column, tableName, columnName) => {
  const columnGraphQLType = drizzleColumnToGraphQLType(column, true);
  const columnArr = new import_graphql3.GraphQLList(new import_graphql3.GraphQLNonNull(columnGraphQLType.type));
  const baseFields = {
    eq: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    ne: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    lt: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    lte: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    gt: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    gte: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    like: { type: import_graphql3.GraphQLString },
    notLike: { type: import_graphql3.GraphQLString },
    ilike: { type: import_graphql3.GraphQLString },
    notIlike: { type: import_graphql3.GraphQLString },
    inArray: { type: columnArr, description: `Array<${columnGraphQLType.description}>` },
    notInArray: { type: columnArr, description: `Array<${columnGraphQLType.description}>` },
    isNull: { type: import_graphql3.GraphQLBoolean },
    isNotNull: { type: import_graphql3.GraphQLBoolean }
  };
  const type = new import_graphql3.GraphQLInputObjectType({
    name: `${pascalize(tableName)}${pascalize(columnName)}Filters`,
    fields: {
      ...baseFields,
      OR: {
        type: new import_graphql3.GraphQLList(
          new import_graphql3.GraphQLNonNull(
            new import_graphql3.GraphQLInputObjectType({
              name: `${pascalize(tableName)}${pascalize(columnName)}filtersOr`,
              fields: {
                ...baseFields
              }
            })
          )
        )
      }
    }
  });
  return type;
};
var filterMap = /* @__PURE__ */ new WeakMap();
var generateTableFilterValuesCached = (table, tableName) => {
  if (filterMap.has(table))
    return filterMap.get(table);
  const columns = (0, import_drizzle_orm3.getTableColumns)(table);
  const columnEntries = Object.entries(columns);
  const remapped = Object.fromEntries(
    columnEntries.map(([columnName, columnDescription]) => [
      columnName,
      {
        type: generateColumnFilterValues(columnDescription, tableName, columnName)
      }
    ])
  );
  filterMap.set(table, remapped);
  return remapped;
};
var fieldMap = /* @__PURE__ */ new WeakMap();
var generateTableSelectTypeFieldsCached = (table) => {
  if (fieldMap.has(table))
    return fieldMap.get(table);
  const columns = (0, import_drizzle_orm3.getTableColumns)(table);
  const columnEntries = Object.entries(columns);
  const remapped = Object.fromEntries(
    columnEntries.map(([columnName, columnDescription]) => [
      columnName,
      drizzleColumnToGraphQLType(columnDescription)
    ])
  );
  fieldMap.set(table, remapped);
  return remapped;
};
var generateTableTypes = (tableName, table, withReturning, relations) => {
  const relationEntries = relations ? Object.entries(relations) : [];
  const relationFields = Object.fromEntries(
    relationEntries.map(([relName, relValue]) => {
      const relTableFields = Object.fromEntries(
        Object.entries((0, import_drizzle_orm3.getTableColumns)(relValue.referencedTable)).map(([columnName, columnDescription]) => [
          columnName,
          drizzleColumnToGraphQLType(columnDescription)
        ])
      );
      const type = new import_graphql3.GraphQLObjectType({
        name: `${pascalize(tableName)}${pascalize(relName)}Relation`,
        fields: relTableFields
      });
      const relationFilterColumns = generateTableFilterValuesCached(relValue.referencedTable, relName);
      const where = new import_graphql3.GraphQLInputObjectType({
        name: `${pascalize(tableName)}${pascalize(relName)}RelationFilters`,
        fields: {
          ...relationFilterColumns,
          OR: {
            type: new import_graphql3.GraphQLList(
              new import_graphql3.GraphQLNonNull(
                new import_graphql3.GraphQLInputObjectType({
                  name: `${pascalize(tableName)}${pascalize(relName)}RelationFiltersOr`,
                  fields: relationFilterColumns
                })
              )
            )
          }
        }
      });
      if ((0, import_drizzle_orm3.is)(relValue, import_drizzle_orm3.One)) {
        return [
          relName,
          {
            type,
            args: {
              where: { type: where }
            }
          }
        ];
      }
      const orderBy = new import_graphql3.GraphQLInputObjectType({
        name: `${pascalize(tableName)}${pascalize(relName)}RelationOrder`,
        fields: Object.fromEntries(
          Object.entries(relValue.referencedTable).map(([columnName, columnDescription]) => [
            columnName,
            { type: innerOrder }
          ])
        )
      });
      return [
        relName,
        {
          type: new import_graphql3.GraphQLNonNull(new import_graphql3.GraphQLList(new import_graphql3.GraphQLNonNull(type))),
          args: {
            where: { type: where },
            orderBy: { type: orderBy },
            offset: { type: import_graphql3.GraphQLInt },
            limit: { type: import_graphql3.GraphQLInt }
          }
        }
      ];
    })
  );
  const tableFields = generateTableSelectTypeFieldsCached(table);
  const columns = (0, import_drizzle_orm3.getTableColumns)(table);
  const columnEntries = Object.entries(columns);
  const insertFields = Object.fromEntries(
    columnEntries.map(([columnName, columnDescription]) => [
      columnName,
      drizzleColumnToGraphQLType(columnDescription, false, true)
    ])
  );
  const updateFields = Object.fromEntries(
    columnEntries.map(([columnName, columnDescription]) => [
      columnName,
      drizzleColumnToGraphQLType(columnDescription, true)
    ])
  );
  const insertInput = new import_graphql3.GraphQLInputObjectType({
    name: `${pascalize(tableName)}InsertInput`,
    fields: insertFields
  });
  const selectSingleOutput = new import_graphql3.GraphQLObjectType({
    name: `${pascalize(tableName)}SelectItem`,
    fields: { ...tableFields, ...relationFields }
  });
  const selectArrOutput = new import_graphql3.GraphQLNonNull(new import_graphql3.GraphQLList(new import_graphql3.GraphQLNonNull(selectSingleOutput)));
  const singleTableItemOutput = withReturning ? new import_graphql3.GraphQLObjectType({
    name: `${pascalize(tableName)}Item`,
    fields: tableFields
  }) : void 0;
  const arrTableItemOutput = withReturning ? new import_graphql3.GraphQLNonNull(new import_graphql3.GraphQLList(new import_graphql3.GraphQLNonNull(singleTableItemOutput))) : void 0;
  const updateInput = new import_graphql3.GraphQLInputObjectType({
    name: `${pascalize(tableName)}UpdateInput`,
    fields: updateFields
  });
  const tableOrder = new import_graphql3.GraphQLInputObjectType({
    name: `${pascalize(tableName)}OrderBy`,
    fields: Object.fromEntries(
      columnEntries.map(([columnName, columnDescription]) => [columnName, { type: innerOrder }])
    )
  });
  const filterColumns = generateTableFilterValuesCached(table, tableName);
  const tableFilters = new import_graphql3.GraphQLInputObjectType({
    name: `${pascalize(tableName)}Filters`,
    fields: {
      ...filterColumns,
      OR: {
        type: new import_graphql3.GraphQLList(
          new import_graphql3.GraphQLNonNull(
            new import_graphql3.GraphQLInputObjectType({
              name: `${pascalize(tableName)}FiltersOr`,
              fields: filterColumns
            })
          )
        )
      }
    }
  });
  const inputs = {
    insertInput,
    updateInput,
    tableOrder,
    tableFilters
  };
  const outputs = withReturning ? {
    selectSingleOutput,
    selectArrOutput,
    singleTableItemOutput,
    arrTableItemOutput
  } : {
    selectSingleOutput,
    selectArrOutput
  };
  return {
    inputs,
    outputs
  };
};
var extractOrderBy = (table, orderArgs) => {
  const res = [];
  for (const [column, config] of Object.entries(orderArgs).sort(
    (a, b) => (b[1]?.priority ?? 0) - (a[1]?.priority ?? 0)
  )) {
    if (!config)
      continue;
    const { direction } = config;
    res.push(direction === "asc" ? (0, import_drizzle_orm3.asc)((0, import_drizzle_orm3.getTableColumns)(table)[column]) : (0, import_drizzle_orm3.desc)((0, import_drizzle_orm3.getTableColumns)(table)[column]));
  }
  return res;
};
var extractFiltersColumn = (column, columnName, operators) => {
  if (!operators.OR?.length)
    delete operators.OR;
  const entries = Object.entries(operators);
  if (operators.OR) {
    if (entries.length > 1) {
      throw new import_graphql3.GraphQLError(`WHERE ${columnName}: Cannot specify both fields and 'OR' in column operators!`);
    }
    const variants2 = [];
    for (const variant of operators.OR) {
      const extracted = extractFiltersColumn(column, columnName, variant);
      if (extracted)
        variants2.push(extracted);
    }
    return variants2.length ? variants2.length > 1 ? (0, import_drizzle_orm3.or)(...variants2) : variants2[0] : void 0;
  }
  const variants = [];
  for (const [operatorName, operatorValue] of entries) {
    if (operatorValue === null || operatorValue === false)
      continue;
    let operator;
    switch (operatorName) {
      case "eq":
        operator = operator ?? import_drizzle_orm3.eq;
      case "ne":
        operator = operator ?? import_drizzle_orm3.ne;
      case "gt":
        operator = operator ?? import_drizzle_orm3.gt;
      case "gte":
        operator = operator ?? import_drizzle_orm3.gte;
      case "lt":
        operator = operator ?? import_drizzle_orm3.lt;
      case "lte":
        operator = operator ?? import_drizzle_orm3.lte;
        const singleValue = remapFromGraphQLCore(operatorValue, column, columnName);
        variants.push(operator(column, singleValue));
        break;
      case "like":
        operator = operator ?? import_drizzle_orm3.like;
      case "notLike":
        operator = operator ?? import_drizzle_orm3.notLike;
      case "ilike":
        operator = operator ?? import_drizzle_orm3.ilike;
      case "notIlike":
        operator = operator ?? import_drizzle_orm3.notIlike;
        variants.push(operator(column, operatorValue));
        break;
      case "inArray":
        operator = operator ?? import_drizzle_orm3.inArray;
      case "notInArray":
        operator = operator ?? import_drizzle_orm3.notInArray;
        if (!operatorValue.length) {
          throw new import_graphql3.GraphQLError(
            `WHERE ${columnName}: Unable to use operator ${operatorName} with an empty array!`
          );
        }
        const arrayValue = operatorValue.map((val) => remapFromGraphQLCore(val, column, columnName));
        variants.push(operator(column, arrayValue));
        break;
      case "isNull":
        operator = operator ?? import_drizzle_orm3.isNull;
      case "isNotNull":
        operator = operator ?? import_drizzle_orm3.isNotNull;
        variants.push(operator(column));
    }
  }
  return variants.length ? variants.length > 1 ? (0, import_drizzle_orm3.and)(...variants) : variants[0] : void 0;
};
var extractFilters = (table, tableName, filters) => {
  if (!filters.OR?.length)
    delete filters.OR;
  const entries = Object.entries(filters);
  if (!entries.length)
    return;
  if (filters.OR) {
    if (entries.length > 1) {
      throw new import_graphql3.GraphQLError(`WHERE ${tableName}: Cannot specify both fields and 'OR' in table filters!`);
    }
    const variants2 = [];
    for (const variant of filters.OR) {
      const extracted = extractFilters(table, tableName, variant);
      if (extracted)
        variants2.push(extracted);
    }
    return variants2.length ? variants2.length > 1 ? (0, import_drizzle_orm3.or)(...variants2) : variants2[0] : void 0;
  }
  const variants = [];
  for (const [columnName, operators] of entries) {
    if (operators === null)
      continue;
    const column = (0, import_drizzle_orm3.getTableColumns)(table)[columnName];
    variants.push(extractFiltersColumn(column, columnName, operators));
  }
  return variants.length ? variants.length > 1 ? (0, import_drizzle_orm3.and)(...variants) : variants[0] : void 0;
};
var extractRelationsParams = (relations, tableSelection, typeName, info) => {
  const fields = {};
  const parsedInfo = (0, import_graphql_parse_resolve_info.parseResolveInfo)(info, {
    deep: true
  });
  const baseField = parsedInfo ? Object.entries(parsedInfo.fieldsByTypeName).find(([key, value]) => key === typeName)?.[1] ?? {} : {};
  for (const [relName, relValue] of Object.entries(relations)) {
    if (!tableSelection.selectionSet)
      continue;
    const node = tableSelection.selectionSet.selections.find(
      (e) => e.kind === import_graphql3.Kind.FIELD && e.name.value === relName
    );
    if (!node)
      continue;
    const refTable = relValue.referencedTable;
    const columns = extractSelectedColumnsFromNode(node, refTable);
    const relationArgs = Object.values(baseField).find((e) => e.name === relName)?.args;
    if (!relationArgs) {
      fields[relName] = {
        columns
      };
      continue;
    }
    const orderBy = relationArgs.orderBy ? extractOrderBy(refTable, relationArgs.orderBy) : void 0;
    const where = relationArgs.where ? extractFilters(refTable, relName, relationArgs?.where) : void 0;
    const offset = relationArgs.offset ?? void 0;
    const limit = relationArgs.limit ?? void 0;
    fields[relName] = {
      columns,
      orderBy,
      where,
      offset,
      limit
    };
  }
  return Object.keys(fields).length ? fields : void 0;
};

// src/util/builders/mysql.ts
var generateSelectArray = (db, tableName, table, relations, orderArgs, filterArgs) => {
  const queryName = `${camelize(tableName)}`;
  const queryBase = db.query[tableName];
  if (!queryBase) {
    throw new Error(
      `Table ${tableName} not found in drizzle instance. Did you forget to pass schema to drizzle constructor?`
    );
  }
  const queryArgs = {
    offset: {
      type: import_graphql4.GraphQLInt
    },
    limit: {
      type: import_graphql4.GraphQLInt
    },
    orderBy: {
      type: orderArgs
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const { offset, limit, orderBy, where } = args;
        const tableSelection = info.operation.selectionSet.selections.find(
          (e) => e.kind === import_graphql4.Kind.FIELD && e.name.value === info.fieldName
        );
        const query = queryBase.findMany({
          columns: extractSelectedColumnsFromNode(tableSelection, table),
          offset,
          limit,
          orderBy: orderBy ? extractOrderBy(table, orderBy) : void 0,
          where: where ? extractFilters(table, tableName, where) : void 0,
          with: relations ? extractRelationsParams(relations, tableSelection, `${pascalize(tableName)}SelectItem`, info) : void 0
        });
        const result = await query;
        return remapToGraphQLArrayOutput(result, relations);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql4.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateSelectSingle = (db, tableName, table, relations, orderArgs, filterArgs) => {
  const queryName = `${camelize(tableName)}Single`;
  const queryBase = db.query[tableName];
  if (!queryBase) {
    throw new Error(
      `Table ${tableName} not found in drizzle instance. Did you forget to pass schema to drizzle constructor?`
    );
  }
  const queryArgs = {
    offset: {
      type: import_graphql4.GraphQLInt
    },
    orderBy: {
      type: orderArgs
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const { offset, orderBy, where } = args;
        const tableSelection = info.operation.selectionSet.selections.find(
          (e) => e.kind === import_graphql4.Kind.FIELD && e.name.value === info.fieldName
        );
        const columns = extractSelectedColumnsFromNode(tableSelection, table);
        const withFields = relations ? extractRelationsParams(relations, tableSelection, `${pascalize(tableName)}SelectItem`, info) : void 0;
        const query = queryBase.findFirst({
          columns,
          offset,
          orderBy: orderBy ? extractOrderBy(table, orderBy) : void 0,
          where: where ? extractFilters(table, tableName, where) : void 0,
          with: withFields
        });
        const result = await query;
        if (!result)
          return void 0;
        return remapToGraphQLSingleOutput(result, relations);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql4.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateInsertArray = (db, tableName, table, baseType) => {
  const queryName = `insertInto${pascalize(tableName)}`;
  const queryArgs = {
    values: {
      type: new import_graphql4.GraphQLNonNull(new import_graphql4.GraphQLList(new import_graphql4.GraphQLNonNull(baseType)))
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const input = remapFromGraphQLArrayInput(args.values, table);
        if (!input.length)
          throw new import_graphql4.GraphQLError("No values were provided!");
        await db.insert(table).values(input);
        return { isSuccess: true };
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql4.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateInsertSingle = (db, tableName, table, baseType) => {
  const queryName = `insertInto${pascalize(tableName)}Single`;
  const queryArgs = {
    values: {
      type: new import_graphql4.GraphQLNonNull(baseType)
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const input = remapFromGraphQLSingleInput(args.values, table);
        await db.insert(table).values(input);
        return { isSuccess: true };
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql4.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateUpdate = (db, tableName, table, setArgs, filterArgs) => {
  const queryName = `update${pascalize(tableName)}`;
  const queryArgs = {
    set: {
      type: new import_graphql4.GraphQLNonNull(setArgs)
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const { where, set } = args;
        const input = remapFromGraphQLSingleInput(set, table);
        if (!Object.keys(input).length)
          throw new import_graphql4.GraphQLError("Unable to update with no values specified!");
        let query = db.update(table).set(input);
        if (where) {
          const filters = extractFilters(table, tableName, where);
          query = query.where(filters);
        }
        await query;
        return { isSuccess: true };
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql4.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateDelete = (db, tableName, table, filterArgs) => {
  const queryName = `deleteFrom${tableName}`;
  const queryArgs = {
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const { where } = args;
        let query = db.delete(table);
        if (where) {
          const filters = extractFilters(table, tableName, where);
          query = query.where(filters);
        }
        await query;
        return { isSuccess: true };
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql4.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateSchemaData = (db, schema) => {
  const rawSchema = schema;
  const schemaEntries = Object.entries(rawSchema);
  const tables = Object.fromEntries(schemaEntries.filter(([key, value]) => (0, import_drizzle_orm4.is)(value, import_mysql_core2.MySqlTable)));
  if (!tables || !Object.keys(tables).length) {
    throw new Error(
      `Unable to extract tables from drizzle instance.
Did you forget to pass tables to graphql schema constructor?`
    );
  }
  const relations = Object.fromEntries(
    schemaEntries.filter(([key, value]) => (0, import_drizzle_orm4.is)(value, import_drizzle_orm4.Relations)).map(([key, value]) => [
      Object.entries(tables).find(
        ([tableName, tableValue]) => tableValue === value.table
      )[0],
      value
    ]).map(([tableName, relValue]) => [
      tableName,
      relValue.config((0, import_drizzle_orm4.createTableRelationsHelpers)(tables[tableName]))
    ])
  );
  const queries = {};
  const mutations = {};
  const gqlSchemaTypes = Object.fromEntries(
    Object.entries(tables).map(([tableName, table]) => [
      tableName,
      generateTableTypes(tableName, table, false, relations[tableName])
    ])
  );
  const mutationReturnType = new import_graphql4.GraphQLObjectType({
    name: `MutationReturn`,
    fields: {
      isSuccess: {
        type: new import_graphql4.GraphQLNonNull(import_graphql4.GraphQLBoolean)
      }
    }
  });
  const inputs = {};
  const outputs = {
    MutationReturn: mutationReturnType
  };
  for (const [tableName, tableTypes] of Object.entries(gqlSchemaTypes)) {
    const { insertInput, updateInput, tableFilters, tableOrder } = tableTypes.inputs;
    const { selectSingleOutput, selectArrOutput } = tableTypes.outputs;
    const selectArrGenerated = generateSelectArray(
      db,
      tableName,
      schema[tableName],
      relations[tableName],
      tableOrder,
      tableFilters
    );
    const selectSingleGenerated = generateSelectSingle(
      db,
      tableName,
      schema[tableName],
      relations[tableName],
      tableOrder,
      tableFilters
    );
    const insertArrGenerated = generateInsertArray(db, tableName, schema[tableName], insertInput);
    const insertSingleGenerated = generateInsertSingle(db, tableName, schema[tableName], insertInput);
    const updateGenerated = generateUpdate(
      db,
      tableName,
      schema[tableName],
      updateInput,
      tableFilters
    );
    const deleteGenerated = generateDelete(db, tableName, schema[tableName], tableFilters);
    queries[selectArrGenerated.name] = {
      type: selectArrOutput,
      args: selectArrGenerated.args,
      resolve: selectArrGenerated.resolver
    };
    queries[selectSingleGenerated.name] = {
      type: selectSingleOutput,
      args: selectSingleGenerated.args,
      resolve: selectSingleGenerated.resolver
    };
    mutations[insertArrGenerated.name] = {
      type: mutationReturnType,
      args: insertArrGenerated.args,
      resolve: insertArrGenerated.resolver
    };
    mutations[insertSingleGenerated.name] = {
      type: mutationReturnType,
      args: insertSingleGenerated.args,
      resolve: insertSingleGenerated.resolver
    };
    mutations[updateGenerated.name] = {
      type: mutationReturnType,
      args: updateGenerated.args,
      resolve: updateGenerated.resolver
    };
    mutations[deleteGenerated.name] = {
      type: mutationReturnType,
      args: deleteGenerated.args,
      resolve: deleteGenerated.resolver
    };
    [insertInput, updateInput, tableFilters, tableOrder].forEach((e) => inputs[e.name] = e);
    outputs[selectSingleOutput.name] = selectSingleOutput;
  }
  return { queries, mutations, inputs, types: outputs };
};

// src/util/builders/pg.ts
var import_drizzle_orm5 = require("drizzle-orm");
var import_pg_core2 = require("drizzle-orm/pg-core");
var import_graphql5 = require("graphql");
var generateSelectArray2 = (db, tableName, table, relations, orderArgs, filterArgs) => {
  const queryName = `${camelize(tableName)}`;
  const queryBase = db.query[tableName];
  if (!queryBase) {
    throw new Error(
      `Table ${tableName} not found in drizzle instance. Did you forget to pass schema to drizzle constructor?`
    );
  }
  const queryArgs = {
    offset: {
      type: import_graphql5.GraphQLInt
    },
    limit: {
      type: import_graphql5.GraphQLInt
    },
    orderBy: {
      type: orderArgs
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const { offset, limit, orderBy, where } = args;
        const tableSelection = info.operation.selectionSet.selections.find(
          (e) => e.kind === import_graphql5.Kind.FIELD && e.name.value === info.fieldName
        );
        const query = queryBase.findMany({
          columns: extractSelectedColumnsFromNode(tableSelection, table),
          offset,
          limit,
          orderBy: orderBy ? extractOrderBy(table, orderBy) : void 0,
          where: where ? extractFilters(table, tableName, where) : void 0,
          with: relations ? extractRelationsParams(relations, tableSelection, `${pascalize(tableName)}SelectItem`, info) : void 0
        });
        const result = await query;
        return remapToGraphQLArrayOutput(result, relations);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql5.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateSelectSingle2 = (db, tableName, table, relations, orderArgs, filterArgs) => {
  const queryName = `${camelize(tableName)}Single`;
  const queryBase = db.query[tableName];
  if (!queryBase) {
    throw new Error(
      `Table ${tableName} not found in drizzle instance. Did you forget to pass schema to drizzle constructor?`
    );
  }
  const queryArgs = {
    offset: {
      type: import_graphql5.GraphQLInt
    },
    orderBy: {
      type: orderArgs
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const { offset, orderBy, where } = args;
        const tableSelection = info.operation.selectionSet.selections.find(
          (e) => e.kind === import_graphql5.Kind.FIELD && e.name.value === info.fieldName
        );
        const query = queryBase.findFirst({
          columns: extractSelectedColumnsFromNode(tableSelection, table),
          offset,
          orderBy: orderBy ? extractOrderBy(table, orderBy) : void 0,
          where: where ? extractFilters(table, tableName, where) : void 0,
          with: relations ? extractRelationsParams(relations, tableSelection, `${pascalize(tableName)}SelectItem`, info) : void 0
        });
        const result = await query;
        if (!result)
          return void 0;
        return remapToGraphQLSingleOutput(result, relations);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql5.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateInsertArray2 = (db, tableName, table, baseType) => {
  const queryName = `insertInto${pascalize(tableName)}`;
  const queryArgs = {
    values: {
      type: new import_graphql5.GraphQLNonNull(new import_graphql5.GraphQLList(new import_graphql5.GraphQLNonNull(baseType)))
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const input = remapFromGraphQLArrayInput(args.values, table);
        if (!input.length)
          throw new import_graphql5.GraphQLError("No values were provided!");
        const columns = extractSelectedColumnsSQLFormat(info, info.fieldName, table);
        const result = await db.insert(table).values(input).returning(columns).onConflictDoNothing();
        return remapToGraphQLArrayOutput(result);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql5.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateInsertSingle2 = (db, tableName, table, baseType) => {
  const queryName = `insertInto${pascalize(tableName)}Single`;
  const queryArgs = {
    values: {
      type: new import_graphql5.GraphQLNonNull(baseType)
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const input = remapFromGraphQLSingleInput(args.values, table);
        const columns = extractSelectedColumnsSQLFormat(info, info.fieldName, table);
        const result = await db.insert(table).values(input).returning(columns).onConflictDoNothing();
        if (!result[0])
          return void 0;
        return remapToGraphQLSingleOutput(result[0]);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql5.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateUpdate2 = (db, tableName, table, setArgs, filterArgs) => {
  const queryName = `update${pascalize(tableName)}`;
  const queryArgs = {
    set: {
      type: new import_graphql5.GraphQLNonNull(setArgs)
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const { where, set } = args;
        const columns = extractSelectedColumnsSQLFormat(info, info.fieldName, table);
        const input = remapFromGraphQLSingleInput(set, table);
        if (!Object.keys(input).length)
          throw new import_graphql5.GraphQLError("Unable to update with no values specified!");
        let query = db.update(table).set(input);
        if (where) {
          const filters = extractFilters(table, tableName, where);
          query = query.where(filters);
        }
        query = query.returning(columns);
        const result = await query;
        return remapToGraphQLArrayOutput(result);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql5.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateDelete2 = (db, tableName, table, filterArgs) => {
  const queryName = `deleteFrom${pascalize(tableName)}`;
  const queryArgs = {
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const { where } = args;
        const columns = extractSelectedColumnsSQLFormat(info, info.fieldName, table);
        let query = db.delete(table);
        if (where) {
          const filters = extractFilters(table, tableName, where);
          query = query.where(filters);
        }
        query = query.returning(columns);
        const result = await query;
        return remapToGraphQLArrayOutput(result);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql5.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateSchemaData2 = (db, schema) => {
  const rawSchema = schema;
  const schemaEntries = Object.entries(rawSchema);
  const tables = Object.fromEntries(schemaEntries.filter(([key, value]) => (0, import_drizzle_orm5.is)(value, import_pg_core2.PgTable)));
  if (!tables || !Object.keys(tables).length) {
    throw new Error(
      `Unable to extract tables from drizzle instance.
Did you forget to pass tables to graphql schema constructor?`
    );
  }
  const relations = Object.fromEntries(
    schemaEntries.filter(([key, value]) => (0, import_drizzle_orm5.is)(value, import_drizzle_orm5.Relations)).map(([key, value]) => [
      Object.entries(tables).find(
        ([tableName, tableValue]) => tableValue === value.table
      )[0],
      value
    ]).map(([tableName, relValue]) => [
      tableName,
      relValue.config((0, import_drizzle_orm5.createTableRelationsHelpers)(tables[tableName]))
    ])
  );
  const queries = {};
  const mutations = {};
  const gqlSchemaTypes = Object.fromEntries(
    Object.entries(tables).map(([tableName, table]) => [
      tableName,
      generateTableTypes(tableName, table, true, relations[tableName])
    ])
  );
  const inputs = {};
  const outputs = {};
  for (const [tableName, tableTypes] of Object.entries(gqlSchemaTypes)) {
    const { insertInput, updateInput, tableFilters, tableOrder } = tableTypes.inputs;
    const { selectSingleOutput, selectArrOutput, singleTableItemOutput, arrTableItemOutput } = tableTypes.outputs;
    const selectArrGenerated = generateSelectArray2(
      db,
      tableName,
      schema[tableName],
      relations[tableName],
      tableOrder,
      tableFilters
    );
    const selectSingleGenerated = generateSelectSingle2(
      db,
      tableName,
      schema[tableName],
      relations[tableName],
      tableOrder,
      tableFilters
    );
    const insertArrGenerated = generateInsertArray2(db, tableName, schema[tableName], insertInput);
    const insertSingleGenerated = generateInsertSingle2(db, tableName, schema[tableName], insertInput);
    const updateGenerated = generateUpdate2(db, tableName, schema[tableName], updateInput, tableFilters);
    const deleteGenerated = generateDelete2(db, tableName, schema[tableName], tableFilters);
    queries[selectArrGenerated.name] = {
      type: selectArrOutput,
      args: selectArrGenerated.args,
      resolve: selectArrGenerated.resolver
    };
    queries[selectSingleGenerated.name] = {
      type: selectSingleOutput,
      args: selectSingleGenerated.args,
      resolve: selectSingleGenerated.resolver
    };
    mutations[insertArrGenerated.name] = {
      type: arrTableItemOutput,
      args: insertArrGenerated.args,
      resolve: insertArrGenerated.resolver
    };
    mutations[insertSingleGenerated.name] = {
      type: singleTableItemOutput,
      args: insertSingleGenerated.args,
      resolve: insertSingleGenerated.resolver
    };
    mutations[updateGenerated.name] = {
      type: arrTableItemOutput,
      args: updateGenerated.args,
      resolve: updateGenerated.resolver
    };
    mutations[deleteGenerated.name] = {
      type: arrTableItemOutput,
      args: deleteGenerated.args,
      resolve: deleteGenerated.resolver
    };
    [insertInput, updateInput, tableFilters, tableOrder].forEach((e) => inputs[e.name] = e);
    outputs[selectSingleOutput.name] = selectSingleOutput;
    outputs[singleTableItemOutput.name] = singleTableItemOutput;
  }
  return { queries, mutations, inputs, types: outputs };
};

// src/util/builders/sqlite.ts
var import_drizzle_orm6 = require("drizzle-orm");
var import_sqlite_core2 = require("drizzle-orm/sqlite-core");
var import_graphql6 = require("graphql");
var generateSelectArray3 = (db, tableName, table, relations, orderArgs, filterArgs) => {
  const queryName = `${camelize(tableName)}`;
  const queryBase = db.query[tableName];
  if (!queryBase) {
    throw new Error(
      `Table ${tableName} not found in drizzle instance. Did you forget to pass schema to drizzle constructor?`
    );
  }
  const queryArgs = {
    offset: {
      type: import_graphql6.GraphQLInt
    },
    limit: {
      type: import_graphql6.GraphQLInt
    },
    orderBy: {
      type: orderArgs
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const { offset, limit, orderBy, where } = args;
        const tableSelection = info.operation.selectionSet.selections.find(
          (e) => e.kind === import_graphql6.Kind.FIELD && e.name.value === info.fieldName
        );
        const query = queryBase.findMany({
          columns: extractSelectedColumnsFromNode(tableSelection, table),
          offset,
          limit,
          orderBy: orderBy ? extractOrderBy(table, orderBy) : void 0,
          where: where ? extractFilters(table, tableName, where) : void 0,
          with: relations ? extractRelationsParams(relations, tableSelection, `${pascalize(tableName)}SelectItem`, info) : void 0
        });
        const result = await query;
        return remapToGraphQLArrayOutput(result, relations);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql6.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateSelectSingle3 = (db, tableName, table, relations, orderArgs, filterArgs) => {
  const queryName = `${camelize(tableName)}Single`;
  const queryBase = db.query[tableName];
  if (!queryBase) {
    throw new Error(
      `Table ${tableName} not found in drizzle instance. Did you forget to pass schema to drizzle constructor?`
    );
  }
  const queryArgs = {
    offset: {
      type: import_graphql6.GraphQLInt
    },
    orderBy: {
      type: orderArgs
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const { offset, orderBy, where } = args;
        const tableSelection = info.operation.selectionSet.selections.find(
          (e) => e.kind === import_graphql6.Kind.FIELD && e.name.value === info.fieldName
        );
        const query = queryBase.findFirst({
          columns: extractSelectedColumnsFromNode(tableSelection, table),
          offset,
          orderBy: orderBy ? extractOrderBy(table, orderBy) : void 0,
          where: where ? extractFilters(table, tableName, where) : void 0,
          with: relations ? extractRelationsParams(relations, tableSelection, `${pascalize(tableName)}SelectItem`, info) : void 0
        });
        const result = await query;
        if (!result)
          return void 0;
        return remapToGraphQLSingleOutput(result, relations);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql6.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateInsertArray3 = (db, tableName, table, baseType) => {
  const queryName = `insertInto${pascalize(tableName)}`;
  const queryArgs = {
    values: {
      type: new import_graphql6.GraphQLNonNull(new import_graphql6.GraphQLList(new import_graphql6.GraphQLNonNull(baseType)))
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const input = remapFromGraphQLArrayInput(args.values, table);
        if (!input.length)
          throw new import_graphql6.GraphQLError("No values were provided!");
        const columns = extractSelectedColumnsSQLFormat(info, info.fieldName, table);
        const result = await db.insert(table).values(input).returning(columns).onConflictDoNothing();
        return remapToGraphQLArrayOutput(result);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql6.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateInsertSingle3 = (db, tableName, table, baseType) => {
  const queryName = `insertInto${pascalize(tableName)}Single`;
  const queryArgs = {
    values: {
      type: new import_graphql6.GraphQLNonNull(baseType)
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const input = remapFromGraphQLSingleInput(args.values, table);
        const columns = extractSelectedColumnsSQLFormat(info, info.fieldName, table);
        const result = await db.insert(table).values(input).returning(columns).onConflictDoNothing();
        if (!result[0])
          return void 0;
        return remapToGraphQLSingleOutput(result[0]);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql6.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateUpdate3 = (db, tableName, table, setArgs, filterArgs) => {
  const queryName = `update${pascalize(tableName)}`;
  const queryArgs = {
    set: {
      type: new import_graphql6.GraphQLNonNull(setArgs)
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const { where, set } = args;
        const columns = extractSelectedColumnsSQLFormat(info, info.fieldName, table);
        const input = remapFromGraphQLSingleInput(set, table);
        if (!Object.keys(input).length)
          throw new import_graphql6.GraphQLError("Unable to update with no values specified!");
        let query = db.update(table).set(input);
        if (where) {
          const filters = extractFilters(table, tableName, where);
          query = query.where(filters);
        }
        query = query.returning(columns);
        const result = await query;
        return remapToGraphQLArrayOutput(result);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql6.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateDelete3 = (db, tableName, table, filterArgs) => {
  const queryName = `deleteFrom${pascalize(tableName)}`;
  const queryArgs = {
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      try {
        const { where } = args;
        const columns = extractSelectedColumnsSQLFormat(info, info.fieldName, table);
        let query = db.delete(table);
        if (where) {
          const filters = extractFilters(table, tableName, where);
          query = query.where(filters);
        }
        query = query.returning(columns);
        const result = await query;
        return remapToGraphQLArrayOutput(result);
      } catch (e) {
        if (typeof e === "object" && typeof e.message === "string") {
          throw new import_graphql6.GraphQLError(e.message);
        }
        throw e;
      }
    },
    args: queryArgs
  };
};
var generateSchemaData3 = (db, schema) => {
  const rawSchema = schema;
  const schemaEntries = Object.entries(rawSchema);
  const tables = Object.fromEntries(schemaEntries.filter(([key, value]) => (0, import_drizzle_orm6.is)(value, import_sqlite_core2.SQLiteTable)));
  if (!tables || !Object.keys(tables).length) {
    throw new Error(
      `Unable to extract tables from drizzle instance.
Did you forget to pass tables to graphql schema constructor?`
    );
  }
  const relations = Object.fromEntries(
    schemaEntries.filter(([key, value]) => (0, import_drizzle_orm6.is)(value, import_drizzle_orm6.Relations)).map(([key, value]) => [
      Object.entries(tables).find(
        ([tableName, tableValue]) => tableValue === value.table
      )[0],
      value
    ]).map(([tableName, relValue]) => [
      tableName,
      relValue.config((0, import_drizzle_orm6.createTableRelationsHelpers)(tables[tableName]))
    ])
  );
  const queries = {};
  const mutations = {};
  const gqlSchemaTypes = Object.fromEntries(
    Object.entries(tables).map(([tableName, table]) => [
      tableName,
      generateTableTypes(tableName, table, true, relations[tableName])
    ])
  );
  const inputs = {};
  const outputs = {};
  for (const [tableName, tableTypes] of Object.entries(gqlSchemaTypes)) {
    const { insertInput, updateInput, tableFilters, tableOrder } = tableTypes.inputs;
    const { selectSingleOutput, selectArrOutput, singleTableItemOutput, arrTableItemOutput } = tableTypes.outputs;
    const selectArrGenerated = generateSelectArray3(
      db,
      tableName,
      schema[tableName],
      relations[tableName],
      tableOrder,
      tableFilters
    );
    const selectSingleGenerated = generateSelectSingle3(
      db,
      tableName,
      schema[tableName],
      relations[tableName],
      tableOrder,
      tableFilters
    );
    const insertArrGenerated = generateInsertArray3(db, tableName, schema[tableName], insertInput);
    const insertSingleGenerated = generateInsertSingle3(db, tableName, schema[tableName], insertInput);
    const updateGenerated = generateUpdate3(
      db,
      tableName,
      schema[tableName],
      updateInput,
      tableFilters
    );
    const deleteGenerated = generateDelete3(db, tableName, schema[tableName], tableFilters);
    queries[selectArrGenerated.name] = {
      type: selectArrOutput,
      args: selectArrGenerated.args,
      resolve: selectArrGenerated.resolver
    };
    queries[selectSingleGenerated.name] = {
      type: selectSingleOutput,
      args: selectSingleGenerated.args,
      resolve: selectSingleGenerated.resolver
    };
    mutations[insertArrGenerated.name] = {
      type: arrTableItemOutput,
      args: insertArrGenerated.args,
      resolve: insertArrGenerated.resolver
    };
    mutations[insertSingleGenerated.name] = {
      type: singleTableItemOutput,
      args: insertSingleGenerated.args,
      resolve: insertSingleGenerated.resolver
    };
    mutations[updateGenerated.name] = {
      type: arrTableItemOutput,
      args: updateGenerated.args,
      resolve: updateGenerated.resolver
    };
    mutations[deleteGenerated.name] = {
      type: arrTableItemOutput,
      args: deleteGenerated.args,
      resolve: deleteGenerated.resolver
    };
    [insertInput, updateInput, tableFilters, tableOrder].forEach((e) => inputs[e.name] = e);
    outputs[selectSingleOutput.name] = selectSingleOutput;
    outputs[singleTableItemOutput.name] = singleTableItemOutput;
  }
  return { queries, mutations, inputs, types: outputs };
};

// src/index.ts
var buildSchema = (db) => {
  const schema = db._.fullSchema;
  let generatorOutput;
  if ((0, import_drizzle_orm7.is)(db, import_mysql_core3.MySqlDatabase)) {
    generatorOutput = generateSchemaData(db, schema);
  } else if ((0, import_drizzle_orm7.is)(db, import_pg_core3.PgDatabase)) {
    generatorOutput = generateSchemaData2(db, schema);
  } else if ((0, import_drizzle_orm7.is)(db, import_sqlite_core3.BaseSQLiteDatabase)) {
    generatorOutput = generateSchemaData3(db, schema);
  } else
    throw new Error("Unknown database instance type");
  const { queries, mutations, inputs, types } = generatorOutput;
  const query = new import_graphql7.GraphQLObjectType({
    name: "Query",
    fields: queries
  });
  const mutation = new import_graphql7.GraphQLObjectType({
    name: "Mutation",
    fields: mutations
  });
  const outputSchema = new import_graphql7.GraphQLSchema({
    query,
    mutation,
    types: [...Object.values(inputs), ...Object.values(types)]
  });
  return { schema: outputSchema, entities: generatorOutput };
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  buildSchema
});
//# sourceMappingURL=index.cjs.map